searchNodes=[{"doc":"ToxiproxyEx is an Elixir API client for the resilience testing tool Toxiproxy.Toxiproxy is a proxy to simulate network and system conditions. The Elixir API aims to make it simple to write tests that ensure your application behaves appropriately under harsh conditions. Before you can use the Elixir library, you need to read the Usage section of the Toxiproxy README.UsageThe Elixir client communicates with the Toxiproxy daemon via HTTP on http://127.0.0.1:8474.For example, to simulate 1000ms latency on a database server you can use the latency toxic with the latency argument (see the Toxiproxy project for a list of all toxics):ToxiproxyEx.get!(:mysql_master) |&gt; ToxiproxyEx.toxic(:latency, latency: 1000) |&gt; ToxiproxyEx.apply!(fn -&gt; Repo.all(Shop) # this took at least 1s end)You can also take an endpoint down for the duration of a function at the TCP level:ToxiproxyEx.get!(:mysql_master) |&gt; ToxiproxyEx.down!(fn -&gt; Repo.all(Shop) # this&#39;ll raise end)If you want to simulate all your Redis instances being down:ToxiproxyEx.grep!(~r/redis/) |&gt; ToxiproxyEx.down!(fn -&gt; # any redis call will fail end)If you want to simulate that your cache server is slow at incoming network (upstream), but fast at outgoing (downstream), you can apply a toxic to just the upstream:ToxiproxyEx.get!(:cache) |&gt; ToxiproxyEx.upstream(:latency, latency: 1000) |&gt; ToxiproxyEx.apply!(fn -&gt; Cache.get(:omg) # will take at least a second end)By default the toxic is applied to the downstream connection, you can be explicit and compose them:ToxiproxyEx.grep!(~r/redis/) |&gt; ToxiproxyEx.upstream(:slow_close, delay: 100) |&gt; ToxiproxyEx.downstream(:latency, jitter: 300) |&gt; ToxiproxyEx.apply!(fn -&gt; # all redises are now slow at responding and closing end)See the Toxiproxy README for a list of toxics.PopulateTo populate Toxiproxy pass the proxy configurations to ToxiproxyEx.populate:ToxiproxyEx.populate!([ %{ name: &quot;mysql_master&quot;, listen: &quot;localhost:21212&quot;, upstream: &quot;localhost:3306&quot;, }, %{ name: &quot;mysql_read_only&quot;, listen: &quot;localhost:21213&quot;, upstream: &quot;localhost:3306&quot;, } ])This will create the proxies passed, or replace the proxies if they already exist in Toxiproxy. It's recommended to do this early as early in boot as possible, see the Toxiproxy README. If you have many proxies, we recommend storing the Toxiproxy configs in a configuration file and deserializing it into ToxiproxyEx.populate.Error HandlingThis library made the choice to use exceptions on the public API methods to signal errors.This was chosen since this is a library meant to be used in testing code only, where you want test cases to fail if your set assumptions are not met. In this sense setting assumptions that will not be met (toxiproxy-server is not running, passing invalid configurations) is considered to be a developer error and should be fixed rather than handled in code.Server ErrorsIf any API interaction with toxiproxy fails, a ServerError will be raised.Client ErrorsIf you miss-configure toxiproxy via the elixir API, an ArgumentError will be raised.","ref":"ToxiproxyEx.html","title":"ToxiproxyEx","type":"module"},{"doc":"Retrieves a list of all proxies from the toxiproxy server.Raises ToxiproxyEx.ServerError if the list of proxies could not be retrieved.ExamplesRetrievs a proxy:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3307&quot;, name: &quot;test_redis_master&quot;) iex&gt; ToxiproxyEx.all!()","ref":"ToxiproxyEx.html#all!/0","title":"ToxiproxyEx.all!/0","type":"function"},{"doc":"Applies all toxics previously defined on the list of proxies during the duration of the given function.Raises ToxiproxyEx.ServerError if the toxics could not be enabled and disabled again on the server.ExamplesAdd toxics and apply them toxic to a single proxy:iex&gt; proxy = ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; proxies = ToxiproxyEx.downstream(proxy, :slow_close, delay: 100) iex&gt; proxies = ToxiproxyEx.downstream(proxies, :latency, jitter: 300) iex&gt; ToxiproxyEx.apply!(proxies, fn -&gt; ...&gt; # All calls to mysql master are now slow at responding and closing. ...&gt; nil ...&gt; end)Add toxics and apply them toxic to a list of proxies:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_follower&quot;) iex&gt; proxies = ToxiproxyEx.all!() iex&gt; proxies = ToxiproxyEx.downstream(proxies, :slow_close, delay: 100) iex&gt; proxies = ToxiproxyEx.downstream(proxies, :latency, jitter: 300) iex&gt; ToxiproxyEx.apply!(proxies, fn -&gt; ...&gt; # All calls to mysql master and follower are now slow at responding and closing. ...&gt; nil ...&gt; end)","ref":"ToxiproxyEx.html#apply!/2","title":"ToxiproxyEx.apply!/2","type":"function"},{"doc":"Creates a proxy on the toxiproxy server.Raises ToxiproxyEx.ServerError if the creation fails.ExamplesCreate a new proxy:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;)Create a new proxy that listens on a specific port:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, listen: &quot;localhost:5555&quot;, name: &quot;test_mysql_master&quot;)Create a new proxy that is disabled by default:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;, enabled: false)","ref":"ToxiproxyEx.html#create!/1","title":"ToxiproxyEx.create!/1","type":"function"},{"doc":"Deletes one or multiple proxies on the toxiproxy server.Raises ToxiproxyEx.ServerError if the deletion fails.ExamplesDestroy a single proxy:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3456&quot;, name: :test_mysql_master) iex&gt; proxy = ToxiproxyEx.get!(:test_mysql_master) iex&gt; ToxiproxyEx.destroy!(proxy) :okDestroy all proxies:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3456&quot;, name: :test_mysql_master) iex&gt; proxies = ToxiproxyEx.all!() iex&gt; ToxiproxyEx.destroy!(proxies) :ok","ref":"ToxiproxyEx.html#destroy!/1","title":"ToxiproxyEx.destroy!/1","type":"function"},{"doc":"Takes down the proxy or the list of proxies during the duration of the given function.Raises ToxiproxyEx.ServerError if the proxy or list of proxies could not have been disabled and enabled again on the server.ExamplesTake down a single proxy:iex&gt; proxy = ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; ToxiproxyEx.down!(proxy, fn -&gt; ...&gt; # Takes mysql master down. ...&gt; nil ...&gt; end)Take down a list of proxies:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_follower&quot;) iex&gt; proxies = ToxiproxyEx.all!() iex&gt; ToxiproxyEx.down!(proxies, fn -&gt; ...&gt; # Takes mysql master and follower down. ...&gt; nil ...&gt; end)","ref":"ToxiproxyEx.html#down!/2","title":"ToxiproxyEx.down!/2","type":"function"},{"doc":"Adds an downstream toxic to the proxy or list of proxies that will be enabled when passed to ToxiproxyEx.apply!/2.ExamplesAdd an downstream toxic to a proxy:iex&gt; proxy = ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; proxies = ToxiproxyEx.downstream(proxy, :latency, latency: 1000) iex&gt; ToxiproxyEx.apply!(proxies, fn -&gt; ...&gt; # Do some testing ...&gt; nil ...&gt; end)Add an downstream toxic to a list of proxies:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3307&quot;, name: &quot;test_mysql_follower&quot;) iex&gt; proxies = ToxiproxyEx.all!() iex&gt; proxies = ToxiproxyEx.downstream(proxies, :latency, latency: 1000) iex&gt; ToxiproxyEx.apply!(proxies, fn -&gt; ...&gt; # Do some testing ...&gt; nil ...&gt; end)","ref":"ToxiproxyEx.html#downstream/3","title":"ToxiproxyEx.downstream/3","type":"function"},{"doc":"Retrieves a proxy from the toxiproxy server.Raises ToxiproxyEx.ServerError if the proxy could not be retrieved. Raises ArgumentError if the proxy does not exist.ExamplesRetrievs a proxy:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; ToxiproxyEx.get!(:test_mysql_master)","ref":"ToxiproxyEx.html#get!/1","title":"ToxiproxyEx.get!/1","type":"function"},{"doc":"Retrieves a list of proxies from the toxiproxy server where the name matches the specificed regex.Raises ToxiproxyEx.ServerError if the list of proxies could not be retrieved. Raises ArgumentError if no proxy matching the specified regex does exist.ExamplesRetrievs a proxy:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3307&quot;, name: &quot;test_mysql_follower&quot;) iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3308&quot;, name: &quot;test_redis_master&quot;) iex&gt; ToxiproxyEx.grep!(~r/master/)","ref":"ToxiproxyEx.html#grep!/1","title":"ToxiproxyEx.grep!/1","type":"function"},{"doc":"Creates proxies based on the passed data. This is usefull to quickly create multiple proxies based on hardcoded value or values read from external sources such as a config file.Nonexisting proxies will be created and existing ones will be updated to match the passed data.Raises ToxiproxyEx.ServerError if the proxies could not have been created on the server.ExamplesCreating proxies:iex&gt; ToxiproxyEx.populate!([ ...&gt; %{name: &quot;test_mysql_master&quot;, upstream: &quot;localhost:5765&quot;}, ...&gt; %{name: &quot;test_mysql_follower&quot;, upstream: &quot;localhost:5766&quot;, enabled: false} ...&gt; ])","ref":"ToxiproxyEx.html#populate!/1","title":"ToxiproxyEx.populate!/1","type":"function"},{"doc":"Re-enables are proxies and disables all toxics on toxiproxy.Raises ToxiproxyEx.ServerError if the server could not have been reset.ExamplesReset toxiproxy:iex&gt; ToxiproxyEx.reset!() :ok","ref":"ToxiproxyEx.html#reset!/0","title":"ToxiproxyEx.reset!/0","type":"function"},{"doc":"Alias for ToxiproxyEx.downstream/3.","ref":"ToxiproxyEx.html#toxic/3","title":"ToxiproxyEx.toxic/3","type":"function"},{"doc":"Alias for ToxiproxyEx.downstream/3.","ref":"ToxiproxyEx.html#toxicate/3","title":"ToxiproxyEx.toxicate/3","type":"function"},{"doc":"Adds an upstream toxic to the proxy or list of proxies that will be enabled when passed to ToxiproxyEx.apply!/2.ExamplesAdd an upstream toxic to a proxy:iex&gt; proxy = ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; proxies = ToxiproxyEx.upstream(proxy, :latency, latency: 1000) iex&gt; ToxiproxyEx.apply!(proxies, fn -&gt; ...&gt; # Do some testing ...&gt; nil ...&gt; end)Add an upstream toxic to a list of proxies:iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3306&quot;, name: &quot;test_mysql_master&quot;) iex&gt; ToxiproxyEx.create!(upstream: &quot;localhost:3307&quot;, name: &quot;test_mysql_follower&quot;) iex&gt; proxies = ToxiproxyEx.all!() iex&gt; proxies = ToxiproxyEx.upstream(proxies, :latency, latency: 1000) iex&gt; ToxiproxyEx.apply!(proxies, fn -&gt; ...&gt; # Do some testing ...&gt; nil ...&gt; end)","ref":"ToxiproxyEx.html#upstream/3","title":"ToxiproxyEx.upstream/3","type":"function"},{"doc":"Gets the version of the running toxiproxy server.Raises ToxiproxyEx.ServerError if the version could not have been fetched from the server.ExamplesGet running toxiproxy version:iex&gt; ToxiproxyEx.version!() &quot;2.1.2&quot;","ref":"ToxiproxyEx.html#version!/0","title":"ToxiproxyEx.version!/0","type":"function"},{"doc":"A hostname or IP address including a port number, e.g. localhost:4539.","ref":"ToxiproxyEx.html#t:host_with_port/0","title":"ToxiproxyEx.host_with_port/0","type":"type"},{"doc":"A proxy that intercepts traffic to and from an upstream server.","ref":"ToxiproxyEx.html#t:proxy/0","title":"ToxiproxyEx.proxy/0","type":"opaque"},{"doc":"A map containing fields required to setup a proxy. Designed to be used with ToxiproxyEx.populate!/1.","ref":"ToxiproxyEx.html#t:proxy_map/0","title":"ToxiproxyEx.proxy_map/0","type":"type"},{"doc":"A collection of proxies.","ref":"ToxiproxyEx.html#t:toxic_collection/0","title":"ToxiproxyEx.toxic_collection/0","type":"opaque"},{"doc":"Raised when communication with the toxiproxy server fails.","ref":"ToxiproxyEx.ServerError.html","title":"ToxiproxyEx.ServerError","type":"exception"},{"doc":"ToxiproxyEx ToxiproxyEx is an Elixir API client for the resilience testing tool Toxiproxy.Toxiproxy is a proxy to simulate network and system conditions. The Elixir API aims to make it simple to write tests that ensure your application behaves appropriately under harsh conditions. Before you can use the Elixir library, you need to read the Usage section of the Toxiproxy README.","ref":"readme.html","title":"ToxiproxyEx","type":"extras"},{"doc":"The Elixir client communicates with the Toxiproxy daemon via HTTP on http://127.0.0.1:8474.For example, to simulate 1000ms latency on a database server you can use the latency toxic with the latency argument (see the Toxiproxy project for a list of all toxics):ToxiproxyEx.get!(:mysql_master) |&gt; ToxiproxyEx.toxic(:latency, latency: 1000) |&gt; ToxiproxyEx.apply!(fn -&gt; Repo.all(Shop) # this took at least 1s end)You can also take an endpoint down for the duration of a function at the TCP level:ToxiproxyEx.get!(:mysql_master) |&gt; ToxiproxyEx.down!(fn -&gt; Repo.all(Shop) # this&#39;ll raise end)If you want to simulate all your Redis instances being down:ToxiproxyEx.grep!(~r/redis/) |&gt; ToxiproxyEx.down!(fn -&gt; # any redis call will fail end)If you want to simulate that your cache server is slow at incoming network (upstream), but fast at outgoing (downstream), you can apply a toxic to just the upstream:ToxiproxyEx.get!(:cache) |&gt; ToxiproxyEx.upstream(:latency, latency: 1000) |&gt; ToxiproxyEx.apply!(fn -&gt; Cache.get(:omg) # will take at least a second end)By default the toxic is applied to the downstream connection, you can be explicit and compose them:ToxiproxyEx.grep!(~r/redis/) |&gt; ToxiproxyEx.upstream(:slow_close, delay: 100) |&gt; ToxiproxyEx.downstream(:latency, jitter: 300) |&gt; ToxiproxyEx.apply!(fn -&gt; # all redises are now slow at responding and closing end)See the Toxiproxy README for a list of toxics.","ref":"readme.html#usage","title":"ToxiproxyEx - Usage","type":"extras"},{"doc":"To populate Toxiproxy pass the proxy configurations to ToxiproxyEx.populate:ToxiproxyEx.populate!([ %{ name: &quot;mysql_master&quot;, listen: &quot;localhost:21212&quot;, upstream: &quot;localhost:3306&quot;, }, %{ name: &quot;mysql_read_only&quot;, listen: &quot;localhost:21213&quot;, upstream: &quot;localhost:3306&quot;, } ])This will create the proxies passed, or replace the proxies if they already exist in Toxiproxy. It's recommended to do this early as early in boot as possible, see the Toxiproxy README. If you have many proxies, we recommend storing the Toxiproxy configs in a configuration file and deserializing it into ToxiproxyEx.populate.","ref":"readme.html#populate","title":"ToxiproxyEx - Populate","type":"extras"},{"doc":"This library made the choice to use exceptions on the public API methods to signal errors.This was chosen since this is a library meant to be used in testing code only, where you want test cases to fail if your set assumptions are not met. In this sense setting assumptions that will not be met (toxiproxy-server is not running, passing invalid configurations) is considered to be a developer error and should be fixed rather than handled in code.Server ErrorsIf any API interaction with toxiproxy fails, a ServerError will be raised.Client ErrorsIf you miss-configure toxiproxy via the elixir API, an ArgumentError will be raised.","ref":"readme.html#error-handling","title":"ToxiproxyEx - Error Handling","type":"extras"},{"doc":"The package can be installed by adding toxiproxy_ex to your list of dependencies in mix.exs:def deps do [ {:toxiproxy_ex, &quot;~&gt; 1.0.0&quot;, only: :test} ] end","ref":"readme.html#installation","title":"ToxiproxyEx - Installation","type":"extras"},{"doc":"Clone the repo and fetch its dependencies:$ git clone https://github.com/jcambass/toxiproxy_ex.git $ cd toxiproxy_ex $ mix deps.getMake sure that you have Toxiproxy installed and start it:$ toxiproxy-serverRun tests:$ mix test","ref":"readme.html#running-tests","title":"ToxiproxyEx - Running tests","type":"extras"}]